/* $Id: xuartlite_intr_example.c,v 1.3 2002/04/02 21:08:05 meinelte Exp $ */
/*****************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
*****************************************************************************/
/******************************************************************************
*
* FILENAME:
*
* xuartlite_intr_example.c
*
* DESCRIPTION:
*
* This file contains a design example using the Uart Lite driver
* (XUartLite) and hardware device using interrupt mode.
*
* NOTES:
*
* The following symbols control some of the code for this example.
*
* USE_XINTC         Define when the Xilinx interrupt controller used
* VXWORKS_PPC       Define when VxWorks on the PowerPC is used
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a jhl  02/13/02 First release
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xbasic_types.h"
#include "xparameters.h"
#include "xintc.h"
#include "xuartlite.h"
#include "xstatus.h"

#define USE_XINTC       /* Use the Xilinx interrupt controller */
//#define VXWORKS_PPC     /* VxWorks on the PowerPC platform only */

#ifdef VXWORKS_PPC

#include "vxWorks.h"
#include "excLib.h"

#endif

/************************** Constant Definitions *****************************/

/* the following constant controls the length of the buffers to be sent
 * and received with the UART,
 */
#define TEST_BUFFER_SIZE            100

/* the following constant determines the interrupt ID of the UART when
 * connected to the Xilinx interrupt controller
 */
#define UART_INTERRUPT_ID           4

/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/

/*****************************************************************************
*
* MACRO:
*
* STATUS_CHECK
*
* DESCRIPTION:
*
* This macro checks the status for functions called. It performs a return
* if the status was not success.  The purpose of this macro is to allow
* centralized status checking.
*
* ARGUMENTS:
*
* Status contains any status from a Xilinx driver.
*
* RETURN VALUE:
*
* None.
*
* NOTES:
*
* Signature: void STATUS_CHECK(XStatus Status)
*
******************************************************************************/
#define STATUS_CHECK(Status)            \
    {                                   \
        if (Status != XST_SUCCESS)      \
        {                               \
            return Status;              \
        }                               \
    }

/************************** Function Prototypes ******************************/

XStatus SetupInterruptSystem(XUartLite *UartLitePtr);

void SendHandler(void *CallBackRef, unsigned int EventData);

void RecvHandler(void *CallBackRef, unsigned int EventData);

/************************** Variable Definitions *****************************/

/* The following variables are shared between non-interrupt processing and
 * interrupt processing such that they must be global.
 */

/* The following buffers are used in this example to send and receive data
 * with the UART.
 */
Xuint8 SendBuffer[TEST_BUFFER_SIZE];
Xuint8 ReceiveBuffer[TEST_BUFFER_SIZE];

/* The following counters are used to determine when the entire buffer has
 * been sent and received.
 */
int TotalReceivedCount;
int TotalSentCount;

/****************************************************************************
*
* FUNCTION:
*
* UartLiteIntrExample
*
* DESCRIPTION:
*
* This function does a minimal test on the UART Lite device and driver as a
* design example.  The purpose of this function is to illustrate
* how to use the XUartLite component.
*
* This function sends data and expects to receive the same data thru the UART.
* The user much provide a physical loopback such that data which is transmitted
* will be received.
*
* This function uses interrupt driver mode of the UART.  The calls to the
* UART Lite driver which are interrupt context, in the handlers, should
* only use the non-blocking calls.
*
* ARGUMENTS:
*
* None.
*
* RETURN VALUE:
*
* XST_SUCCESS if successful, otherwise XST_FAILURE.
*
* NOTES:
*
* This function contains an infinite loop such that if interrupts are not
* working it may never return.
*
****************************************************************************/

XStatus UartLiteIntrExample()
{
    XUartLite UartLite;
    XStatus Status;
    int Index;

    /* Initialize the UART Lite driver so that it's ready to use,
     * specify the device ID that is generated in xparameters.h
     */
    Status = XUartLite_Initialize(&UartLite, XPAR_MYUART_DEVICE_ID);
    STATUS_CHECK(Status);

    /* Perform a self-test to ensure that the hardware was built
     * correctly
     */
    Status = XUartLite_SelfTest(&UartLite);
    STATUS_CHECK(Status);

    /* Connect the UART to the interrupt subsystem such that interrupts can
     * occur.  This function is application specific.
     */
    Status = SetupInterruptSystem(&UartLite);
    STATUS_CHECK(Status);

    /* Setup the handlers for the UART that will be called from the interrupt
     * context when data has been sent and received, specify a pointer to the
     * UART driver instance as the callback reference so the handlers are able
     * to access the instance data
     */
    XUartLite_SetSendHandler(&UartLite, SendHandler, &UartLite);
    XUartLite_SetRecvHandler(&UartLite, RecvHandler, &UartLite);

    /* Enable the interrupt of the UART so interrupts will occur. */

    XUartLite_EnableInterrupt(&UartLite);

    /* Initialize the send buffer bytes with a pattern to send and the
     * the receive buffer bytes to zero to allow the receive data to be
     * verified
     */
    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        SendBuffer[Index] = 'A'+Index;
        ReceiveBuffer[Index] = 0;
    }

    /*
     * Start receiving data before sending it since there is a loopback.
     */
    XUartLite_Recv(&UartLite, ReceiveBuffer, TEST_BUFFER_SIZE);

    /*
     * Send the buffer using the UART
     */
    XUartLite_Send(&UartLite, SendBuffer, TEST_BUFFER_SIZE);

    /* Wait for the entire buffer to be received, letting the interrupt
     * processing work in the background, this function may get locked
     * up in this loop if the interrupts are not working correctly.
     */
    while ((TotalReceivedCount != TEST_BUFFER_SIZE) ||
           (TotalSentCount != TEST_BUFFER_SIZE))
    {
    }

    /* Verify the entire receive buffer was successfully received */

    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        if (ReceiveBuffer[Index] != SendBuffer[Index])
        {
            return XST_FAILURE;
        }
    }

    return XST_SUCCESS;
}

/****************************************************************************
*
* FUNCTION:
*
* SendHandler
*
* DESCRIPTION:
*
* This function is the handler which performs processing to send data to the
* UART.  It is called from an interrupt context such that the amount of
* processing performed should be minimized.  It is called when the transmit
* FIFO of the UART is empty and more data can be sent thru the UART.
*
* This handler provides an example of how to handle data for the UART, but
* is application specific.
*
* ARGUMENTS:
*
* CallBackRef contains a callback reference from the driver, in this case
* it is the instance pointer for the UART driver.
*
* EventData contains the number of bytes sent or received for sent and
* receive events.
*
* RETURN VALUE:
*
* None
*
* NOTES:
*
*
****************************************************************************/
void SendHandler(void *CallBackRef, unsigned int EventData)
{
    TotalSentCount = EventData;
}

/****************************************************************************
*
* FUNCTION:
*
* RecvHandler
*
* DESCRIPTION:
*
* This function is the handler which performs processing to receive data from
* the UART.  It is called from an interrupt context such that the amount of
* processing performed should be minimized.  It is called when any data is
* present in the receive FIFO of the UART such that the data can be retrieved
* from the UART.  The amount of data present in the FIFO is not known when
* this function is called.
*
* This handler provides an example of how to handle data for the UART, but
* is application specific.
*
* ARGUMENTS:
*
*
* CallBackRef contains a callback reference from the driver, in this case
* it is the instance pointer for the UART driver.
*
* EventData contains the number of bytes sent or received for sent and
* receive events.
*
* RETURN VALUE:
*
* None
*
* NOTES:
*
*
****************************************************************************/
void RecvHandler(void *CallBackRef, unsigned int EventData)
{
    TotalReceivedCount = EventData;
}

/****************************************************************************
*
* FUNCTION:
*
* SetupInterruptSystem
*
* DESCRIPTION:
*
* This function setups the interrupt system such that interrupts can occur
* for the UART.  This function is application specific since the actual
* system may or may not have an interrupt controller.  The UART could be
* directly connected to a processor without an interrupt controller.  The
* user should modify this function to fit the application.
*
* ARGUMENTS:
*
* UartLitePtr contains a pointer to the instance of the Uart component which
* is going to be connected to the interrupt controller.
*
* RETURN VALUE:
*
* XST_SUCCESS if successful, otherwise XST_FAILURE.
*
* NOTES:
*
*
****************************************************************************/
XStatus SetupInterruptSystem(XUartLite *UartLitePtr)
{

#ifdef USE_XINTC        /* use the Xilinx interrupt controller */

    XIntc InterruptController;
    XStatus Status;


    /* Initialize the interrupt controller driver so that
     * it's ready to use, specify the device ID that is generated in
     * xparameters.h
     */
    Status = XIntc_Initialize(&InterruptController, XPAR_INTC_DEVICE_ID);
    STATUS_CHECK(Status);

#ifdef VXWORKS_PPC      /* VxWorks on the PowerPC platform only */

    /* connect the handler to the VxWorks non-critical external interrupt */

    excIntConnect((VOIDFUNCPTR *)_EXC_OFF_INTR,
                   (VOIDFUNCPTR)XIntc_VoidInterruptHandler);
#endif

    /* Connect a device driver handler that will be called when an interrupt
     * for the device occurs, the device driver handler performs the specific
     * interrupt processing for the device
     */
    Status = XIntc_Connect(&InterruptController, UART_INTERRUPT_ID,
                           (XInterruptHandler)XUartLite_InterruptHandler,
                           (void *)UartLitePtr);
    STATUS_CHECK(Status);

    /* Start the interrupt controller such that interrupts are enabled for
     * all devices that cause interrupts, specific real mode so that
     * the UART can cause interrupts thru the interrupt controller.
     */
    Status = XIntc_Start(&InterruptController, XIN_REAL_MODE);
    STATUS_CHECK(Status);

    /* Enable the interrupt for the UART */

    XIntc_Enable(&InterruptController, UART_INTERRUPT_ID);

#endif

    return XST_SUCCESS;
}
