/* $Id: xuartlite_polled_example.c,v 1.3 2002/04/02 21:08:53 meinelte Exp $ */
/*****************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
*****************************************************************************/
/******************************************************************************
*
* FILENAME:
*
* xuartlite_polled_example.c
*
* DESCRIPTION:
*
* This file contains a design example using the Uart Lite driver
* (XUartLite) and hardware device using polled mode.
*
* NOTES:
*
* None.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a jhl  02/13/02 First release
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xbasic_types.h"
#include "xparameters.h"
#include "xuartlite.h"
#include "xstatus.h"

/************************** Constant Definitions *****************************/

/* the following constant controls the length of the buffers to be sent
 * and received with the UART, this constant must be 16 bytes or less since
 * this is a single threaded non-interrupt driven example such that the
 * entire buffer will fit into the transmit and receive FIFOs of the UART
 */
#define TEST_BUFFER_SIZE 16

/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/****************************************************************************
*
* MACRO:
*
* STATUS_CHECK
*
* DESCRIPTION:
*
* This macro checks the status for functions called. It performs a return
* if the status was not success.  The purpose of this macro is to allow
* centralized status checking.
*
* ARGUMENTS:
*
* Status contains any status from a Xilinx driver.
*
* RETURN VALUE:
*
* None.
*
* NOTES:
*
* Signature: void STATUS_CHECK(XStatus Status)
*
****************************************************************************/
#define STATUS_CHECK(Status)            \
    {                                   \
        if (Status != XST_SUCCESS)      \
        {                               \
            return Status;              \
        }                               \
    }

/************************** Function Prototypes ******************************/


/************************** Variable Definitions *****************************/

/****************************************************************************
*
* FUNCTION:
*
* UartLitePolledExample
*
* DESCRIPTION:
*
* This function does a minimal test on the UART Lite device and driver as a
* design example.  The purpose of this function is to illustrate
* how to use the XUartLite component.
*
* This function sends data and expects to receive the data thru the UART
* such that a  physical loopback must be done with the tranmit and receive
* signals of the UART.
*
* This function polls the UART and does not require the use of interrupts.
*
* ARGUMENTS:
*
* None.
*
* RETURN VALUE:
*
* XST_SUCCESS if successful, otherwise XST_FAILURE.
*
* NOTES:
*
* This function calls the UART driver functions in a blocking mode such that
* if the transmit data does not loopback to the receive, this function may
* not return.
*
****************************************************************************/

XStatus UartLitePolledExample()
{
    XUartLite UartLite;
    XStatus Status;
    unsigned int SentCount;
    unsigned int ReceivedCount;
    int Index;
    Xuint8 SendBuffer[TEST_BUFFER_SIZE];
    Xuint8 RecvBuffer[TEST_BUFFER_SIZE];


    /* Initialize the UART Lite driver so that it's ready to use,
     * specify the device ID that is generated in xparameters.h
     */
    Status = XUartLite_Initialize(&UartLite, XPAR_MYUART_DEVICE_ID);
    STATUS_CHECK(Status);

    /* Perform a self-test to ensure that the hardware was built
     * correctly
     */
    Status = XUartLite_SelfTest(&UartLite);
    STATUS_CHECK(Status);

    /* Initialize the send buffer bytes with a pattern to send and the
     * the receive buffer bytes to zero
     */
    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        SendBuffer[Index] = 'a'+Index;
        RecvBuffer[Index] = 0;
    }

    /* Send the buffer thru the UART waiting til the data can be sent (block),
     * if the specified number of bytes was not sent successfully, then
     * an error occurred
     */
    SentCount = XUartLite_Send(&UartLite, SendBuffer,
                               TEST_BUFFER_SIZE);
    if (SentCount != TEST_BUFFER_SIZE)
    {
        return XST_FAILURE;
    }

    /* Wait while the UART is sending the data so that we are guaranteed
     * to get the data the 1st time we call receive, otherwise this function
     * may enter receive before the data has arrived
     */
    while (XUartLite_IsSending(&UartLite))
    {
    }

    /* Receive the buffer from the UART waiting til there's data by (block),
     * if the specified number of bytes was not received successfully, then
     * an error occurred
     */

    /* For the Xilinx idiots : XUartLite_Recv() is NON BLOCKING */
    for(Index=0;Index<1000000;Index++);

    ReceivedCount = XUartLite_Recv(&UartLite, RecvBuffer,
                                   TEST_BUFFER_SIZE);
    if (ReceivedCount != TEST_BUFFER_SIZE)
    {

        return XST_FAILURE;

    }

    /* Check the receive buffer data against the send buffer and verify the
     * data was correctly received
     */
    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        if (SendBuffer[Index] != RecvBuffer[Index])
        {
            return XST_FAILURE;
        }
    }

    return XST_SUCCESS;
}
